@@ -26,6 +26,7 @@
 #include "src/cast.h"
 #include "src/filenames.h"
 #include "src/ir.h"
+#include "src/literal.h"
 #include "src/stream.h"
 #include "src/string-view.h"
 
@@ -110,7 +111,7 @@ void BinaryWriterSpec::WriteEscapedString(string_view s) {
   json_stream_->WriteChar('"');
   for (size_t i = 0; i < s.length(); ++i) {
     uint8_t c = s[i];
-    if (c < 0x20 || c == '\\' || c == '"' || c > 0x7f) {
+    if (c < 0x20 || c == '\\' || c == '"') {
       json_stream_->Writef("\\u%04x", c);
     } else {
       json_stream_->WriteChar(c);
@@ -200,6 +201,27 @@ void BinaryWriterSpec::WriteConst(const Const& const_) {
       break;
     }
 
+    case Type::Nullref:
+    case Type::Funcref:
+    case Type::Anyref: {
+      WriteString("ref");
+      WriteSeparator();
+      WriteKey("value");
+      int64_t ref_bits = static_cast<int64_t>(const_.ref_bits);
+      json_stream_->Writef("\"%" PRIu64 "\"", ref_bits);
+      break;
+    }
+
+    case Type::V128: {
+      WriteString("v128");
+      WriteSeparator();
+      WriteKey("value");
+      char buffer[128];
+      WriteUint128(buffer, 128, const_.v128_bits);
+      json_stream_->Writef("\"%s\"", buffer);
+      break;
+    }
+
     default:
       assert(0);
   }
@@ -490,6 +512,9 @@ void BinaryWriterSpec::WriteCommands() {
         WriteSeparator();
         WriteAction(*assert_exhaustion_command->action);
         WriteSeparator();
+        WriteKey("text");
+        WriteEscapedString(assert_exhaustion_command->text);
+        WriteSeparator();
         WriteKey("expected");
         WriteActionResultType(*assert_exhaustion_command->action);
         break;
