@@ -266,6 +266,7 @@ class CWriter {
   void Write(const TernaryExpr&);
   void Write(const SimdLaneOpExpr&);
   void Write(const SimdShuffleOpExpr&);
+  void Write(const LoadSplatExpr&);
 
   const WriteCOptions& options_;
   const Module* module_ = nullptr;
@@ -335,7 +336,7 @@ static const char* s_global_symbols[] = {
     "uint_fast64_t", "UINT_FAST8_MAX", "uint_fast8_t", "UINT_LEAST16_MAX",
     "uint_least16_t", "UINT_LEAST32_MAX", "uint_least32_t", "UINT_LEAST64_MAX",
     "uint_least64_t", "UINT_LEAST8_MAX", "uint_least8_t", "UINTMAX_C",
-    "UINTMAX_MAX", "uintmax_t", "UINTPTR_MAX", "uintptr_t", "UNT8_C",
+    "UINTMAX_MAX", "uintmax_t", "UINTPTR_MAX", "uintptr_t", "UINT8_C",
     "WCHAR_MAX", "WCHAR_MIN", "WINT_MAX", "WINT_MIN",
 
     // string.h
@@ -1656,6 +1657,10 @@ void CWriter::Write(const ExprList& exprs) {
         break;
       }
 
+      case ExprType::LoadSplat:
+        Write(*cast<LoadSplatExpr>(&expr));
+        break;
+
       case ExprType::Unreachable:
         Write("UNREACHABLE;", Newline());
         return;
@@ -2256,6 +2261,20 @@ void CWriter::Write(const SimdShuffleOpExpr& expr) {
   PushType(result_type);
 }
 
+void CWriter::Write(const LoadSplatExpr& expr) {
+  assert(module_->memories.size() == 1);
+  Memory* memory = module_->memories[0];
+
+  Type result_type = expr.opcode.GetResultType();
+  Write(StackVar(0, result_type), " = ", expr.opcode.GetName(), "(",
+        ExternalPtr(memory->name), ", (u64)(", StackVar(0));
+  if (expr.offset != 0)
+    Write(" + ", expr.offset);
+  Write("));", Newline());
+  DropTypes(1);
+  PushType(result_type);
+}
+
 void CWriter::WriteCHeader() {
   stream_ = h_stream_;
   std::string guard = GenerateHeaderGuard();
