@@ -102,6 +102,14 @@
 
 struct v128 {
   uint32_t v[4];
+
+  bool operator==(const v128& other) const {
+    return v[0] == other.v[0] &&
+           v[1] == other.v[1] &&
+           v[2] == other.v[2] &&
+           v[3] == other.v[3];
+  }
+  bool operator!=(const v128& other) const { return !(*this == other); }
 };
 
 namespace wabt {
@@ -199,18 +207,19 @@ struct Location {
 
 // Matches binary format, do not change.
 enum class Type : int32_t {
-  I32 = -0x01,        // 0x7f
-  I64 = -0x02,        // 0x7e
-  F32 = -0x03,        // 0x7d
-  F64 = -0x04,        // 0x7c
-  V128 = -0x05,       // 0x7b
-  Funcref = -0x10,    // 0x70
-  Anyref = -0x11,     // 0x6f
-  ExceptRef = -0x18,  // 0x68
-  Func = -0x20,       // 0x60
-  Void = -0x40,       // 0x40
-  ___ = Void,         // Convenient for the opcode table in opcode.h
-  Any = 0,            // Not actually specified, but useful for type-checking
+  I32 = -0x01,      // 0x7f
+  I64 = -0x02,      // 0x7e
+  F32 = -0x03,      // 0x7d
+  F64 = -0x04,      // 0x7c
+  V128 = -0x05,     // 0x7b
+  Funcref = -0x10,  // 0x70
+  Anyref = -0x11,   // 0x6f
+  Exnref = -0x18,   // 0x68
+  Func = -0x20,     // 0x60
+  Void = -0x40,     // 0x40
+  ___ = Void,       // Convenient for the opcode table in opcode.h
+  Any = 0,          // Not actually specified, but useful for type-checking
+  Nullref = 1,      // Not actually specified, but useful for anyref type-checking
 };
 typedef std::vector<Type> TypeVector;
 
@@ -258,10 +267,18 @@ enum class SymbolType {
   Event = 4,
 };
 
-#define WABT_SYMBOL_FLAG_UNDEFINED 0x10
+enum class ComdatType {
+  Data = 0x0,
+  Function = 0x1,
+};
+
 #define WABT_SYMBOL_MASK_VISIBILITY 0x4
 #define WABT_SYMBOL_MASK_BINDING 0x3
-#define WASM_SYMBOL_EXPLICIT_NAME 0x40
+#define WABT_SYMBOL_FLAG_UNDEFINED 0x10
+#define WABT_SYMBOL_FLAG_EXPORTED 0x20
+#define WABT_SYMBOL_FLAG_EXPLICIT_NAME 0x40
+#define WABT_SYMBOL_FLAG_NO_STRIP 0x80
+#define WABT_SYMBOL_FLAG_MAX 0xff
 
 enum class SymbolVisibility {
   Default = 0,
@@ -364,8 +381,8 @@ static WABT_INLINE const char* GetTypeName(Type type) {
       return "funcref";
     case Type::Func:
       return "func";
-    case Type::ExceptRef:
-      return "except_ref";
+    case Type::Exnref:
+      return "exnref";
     case Type::Void:
       return "void";
     case Type::Any:
@@ -397,6 +414,9 @@ static WABT_INLINE TypeVector GetInlineTypeVector(Type type) {
     case Type::F32:
     case Type::F64:
     case Type::V128:
+    case Type::Funcref:
+    case Type::Anyref:
+    case Type::Exnref:
       return TypeVector(&type, &type + 1);
 
     default:
