@@ -59,7 +59,9 @@ class BinaryReaderStrip : public BinaryReaderNop {
     return true;
   }
 
-  Result BeginSection(BinarySection section_type, Offset size) override {
+  Result BeginSection(Index section_index,
+                      BinarySection section_type,
+                      Offset size) override {
     if (section_type == BinarySection::Custom) {
       return Result::Ok;
     }
