@@ -169,9 +169,16 @@ Result TypeChecker::CheckTypeStackEnd(const char* desc) {
 }
 
 Result TypeChecker::CheckType(Type actual, Type expected) {
-  return (expected == actual || expected == Type::Any || actual == Type::Any)
-             ? Result::Ok
-             : Result::Error;
+  if (expected == actual || expected == Type::Any || actual == Type::Any) {
+    return Result::Ok;
+  }
+  if (expected == Type::Anyref && (actual == Type::Funcref || actual == Type::Nullref)) {
+    return Result::Ok;
+  }
+  if ((expected == Type::Funcref || expected == Type::Anyref) && actual == Type::Nullref) {
+    return Result::Ok;
+  }
+  return Result::Error;
 }
 
 Result TypeChecker::CheckTypes(const TypeVector& actual,
@@ -385,7 +392,7 @@ Result TypeChecker::OnBrIf(Index depth) {
 }
 
 Result TypeChecker::OnBrOnExn(Index depth, const TypeVector& types) {
-  Result result = PopAndCheck1Type(Type::ExceptRef, "br_on_exn");
+  Result result = PopAndCheck1Type(Type::Exnref, "br_on_exn");
   Label* label;
   CHECK_RESULT(GetLabel(depth, &label));
   if (Failed(CheckTypes(types, label->br_types()))) {
@@ -394,7 +401,7 @@ Result TypeChecker::OnBrOnExn(Index depth, const TypeVector& types) {
                TypesToString(types).c_str());
     result = Result::Error;
   }
-  PushType(Type::ExceptRef);
+  PushType(Type::Exnref);
   return result;
 }
 
@@ -481,7 +488,7 @@ Result TypeChecker::OnCatch() {
   ResetTypeStackToLabel(label);
   label->label_type = LabelType::Catch;
   label->unreachable = false;
-  PushType(Type::ExceptRef);
+  PushType(Type::Exnref);
   return result;
 }
 
@@ -631,7 +638,7 @@ Result TypeChecker::OnTableInit(uint32_t segment) {
 
 Result TypeChecker::OnTableGet(Index segment) {
   Result result = PopAndCheck1Type(Type::I32, "table.get");
-  PushType(Type::Anyref); // TODO: should be the table's type
+  PushType(Type::Nullref); // TODO: should be the table's type
   return result;
 }
 
@@ -641,7 +648,7 @@ Result TypeChecker::OnTableSet(Index segment) {
 }
 
 Result TypeChecker::OnTableGrow(Index segment) {
-  Result result = PopAndCheck2Types(Type::I32, Type::Anyref, "table.grow");
+  Result result = PopAndCheck2Types(Type::Anyref, Type::I32, "table.grow");
   PushType(Type::I32);
   return result;
 }
@@ -652,7 +659,7 @@ Result TypeChecker::OnTableSize(Index segment) {
 }
 
 Result TypeChecker::OnRefNullExpr() {
-  PushType(Type::Anyref);
+  PushType(Type::Nullref);
   return Result::Ok;
 }
 
@@ -663,7 +670,7 @@ Result TypeChecker::OnRefIsNullExpr() {
 }
 
 Result TypeChecker::OnRethrow() {
-  Result result = PopAndCheck1Type(Type::ExceptRef, "rethrow");
+  Result result = PopAndCheck1Type(Type::Exnref, "rethrow");
   CHECK_RESULT(SetUnreachable());
   return result;
 }
