@@ -104,6 +104,7 @@ class Validator : public ExprVisitor::Delegate {
   Result OnTernaryExpr(TernaryExpr*) override;
   Result OnSimdLaneOpExpr(SimdLaneOpExpr*) override;
   Result OnSimdShuffleOpExpr(SimdShuffleOpExpr*) override;
+  Result OnLoadSplatExpr(LoadSplatExpr*) override;
 
  private:
   struct ActionResult {
@@ -411,7 +412,7 @@ void Validator::CheckType(const Location* loc,
                           Type actual,
                           Type expected,
                           const char* desc) {
-  if (expected != actual) {
+  if (Failed(TypeChecker::CheckType(actual, expected))) {
     PrintError(loc, "type mismatch at %s. got %s, expected %s", desc,
                GetTypeName(actual), GetTypeName(expected));
   }
@@ -423,7 +424,7 @@ void Validator::CheckTypeIndex(const Location* loc,
                                const char* desc,
                                Index index,
                                const char* index_kind) {
-  if (expected != actual && expected != Type::Any && actual != Type::Any) {
+  if (Failed(TypeChecker::CheckType(actual, expected))) {
     PrintError(
         loc, "type mismatch for %s %" PRIindex " of %s. got %s, expected %s",
         index_kind, index, desc, GetTypeName(actual), GetTypeName(expected));
@@ -982,6 +983,15 @@ Result Validator::OnSimdShuffleOpExpr(SimdShuffleOpExpr* expr) {
   return Result::Ok;
 }
 
+Result Validator::OnLoadSplatExpr(LoadSplatExpr* expr) {
+  expr_loc_ = &expr->loc;
+  CheckHasMemory(&expr->loc, expr->opcode);
+  CheckAlign(&expr->loc, expr->align,
+             get_opcode_natural_alignment(expr->opcode));
+  typechecker_.OnLoad(expr->opcode);
+  return Result::Ok;
+}
+
 void Validator::CheckFuncSignature(const Location* loc,
                                    const FuncDeclaration& decl) {
   if (decl.has_func_type) {
@@ -1059,6 +1069,10 @@ void Validator::CheckConstInitExpr(const Location* loc,
         break;
       }
 
+      case ExprType::RefNull:
+        type = Type::Nullref;
+        break;
+      
       default:
         PrintConstExprError(loc, desc);
         return;
