@@ -396,10 +396,11 @@ TokenType WastParser::Peek(size_t n) {
     if (cur.token_type() != TokenType::LparAnn) {
       tokens_.push_back(cur);
     } else {
-      // Custom annotation. For now, discard until matching Rpar
+      // Custom annotation. For now, discard until matching Rpar.
       if (!options_->features.annotations_enabled()) {
         Error(cur.loc, "annotations not enabled: %s", cur.to_string().c_str());
-        return TokenType::Invalid;
+        tokens_.push_back(Token(cur.loc, TokenType::Invalid));
+        continue;
       }
       int indent = 1;
       while (indent > 0) {
@@ -587,17 +588,26 @@ bool WastParser::ParseVarOpt(Var* out_var, Var default_var) {
 
 Result WastParser::ParseOffsetExpr(ExprList* out_expr_list) {
   WABT_TRACE(ParseOffsetExpr);
-  if (MatchLpar(TokenType::Offset)) {
-    CHECK_RESULT(ParseTerminatingInstrList(out_expr_list));
-    EXPECT(Rpar);
-  } else if (PeekMatchExpr()) {
-    CHECK_RESULT(ParseExpr(out_expr_list));
-  } else {
+  if (!ParseOffsetExprOpt(out_expr_list)) {
     return ErrorExpected({"an offset expr"}, "(i32.const 123)");
   }
   return Result::Ok;
 }
 
+bool WastParser::ParseOffsetExprOpt(ExprList* out_expr_list) {
+  WABT_TRACE(ParseOffsetExprOpt);
+  if (MatchLpar(TokenType::Offset)) {
+    CHECK_RESULT(ParseTerminatingInstrList(out_expr_list));
+    EXPECT(Rpar);
+    return true;
+  } else if (PeekMatchExpr()) {
+    CHECK_RESULT(ParseExpr(out_expr_list));
+    return true;
+  } else {
+    return false;
+  }
+}
+
 Result WastParser::ParseTextList(std::vector<uint8_t>* out_data) {
   WABT_TRACE(ParseTextList);
   if (!ParseTextListOpt(out_data)) {
@@ -698,6 +708,22 @@ Result WastParser::ParseRefType(Type* out_type) {
   return Result::Ok;
 }
 
+bool WastParser::ParseRefTypeOpt(Type* out_type) {
+  WABT_TRACE(ParseRefTypeOpt);
+  if (!PeekMatch(TokenType::ValueType)) {
+    return false;
+  }
+
+  Token token = Consume();
+  Type type = token.type();
+  if (type == Type::Anyref && !options_->features.reference_types_enabled()) {
+    return false;
+  }
+
+  *out_type = type;
+  return true;
+}
+
 Result WastParser::ParseQuotedText(std::string* text) {
   WABT_TRACE(ParseQuotedText);
   if (!PeekMatch(TokenType::Text)) {
@@ -889,13 +915,12 @@ Result WastParser::ParseDataModuleField(Module* module) {
   ParseBindVarOpt(&name);
   auto field = MakeUnique<DataSegmentModuleField>(loc, name);
 
-  if (Peek() == TokenType::Passive) {
-    Consume();
-    field->data_segment.passive = true;
-  } else {
-    ParseVarOpt(&field->data_segment.memory_var, Var(0, loc));
+  if (ParseVarOpt(&field->data_segment.memory_var, Var(0, loc))) {
     CHECK_RESULT(ParseOffsetExpr(&field->data_segment.offset));
+  } else if (!ParseOffsetExprOpt(&field->data_segment.offset)) {
+    field->data_segment.passive = true;
   }
+
   ParseTextListOpt(&field->data_segment.data);
   EXPECT(Rpar);
   module->AppendField(std::move(field));
@@ -911,10 +936,8 @@ Result WastParser::ParseElemModuleField(Module* module) {
   ParseBindVarOpt(&name);
   auto field = MakeUnique<ElemSegmentModuleField>(loc, name);
 
-  if (Peek() == TokenType::Passive) {
-    Consume();
+  if (ParseRefTypeOpt(&field->elem_segment.elem_type)) {
     field->elem_segment.passive = true;
-    CHECK_RESULT(ParseRefType(&field->elem_segment.elem_type));
     // Parse a potentially empty sequence of ElemExprs.
     while (true) {
       Var var;
@@ -1575,9 +1598,12 @@ Result WastParser::ParsePlainInstr(std::unique_ptr<Expr>* out_expr) {
       break;
     }
 
-    case TokenType::Binary:
-      out_expr->reset(new BinaryExpr(Consume().opcode(), loc));
+    case TokenType::Binary: {
+      Token token = Consume();
+      ErrorUnlessOpcodeEnabled(token);
+      out_expr->reset(new BinaryExpr(token.opcode(), loc));
       break;
+    }
 
     case TokenType::Compare:
       out_expr->reset(new CompareExpr(Consume().opcode(), loc));
@@ -1825,12 +1851,10 @@ Result WastParser::ParseSimdV128Const(Const* const_, TokenType token_type) {
     Location loc = GetLocation();
 
     // Check that the lane literal type matches the element type of the v128:
-    if (integer) {
-      if (!(PeekMatch(TokenType::Int) || PeekMatch(TokenType::Nat))) {
+    if (!PeekMatch(TokenType::Int) && !PeekMatch(TokenType::Nat)) {
+      if (integer) {
         return ErrorExpected({"a Nat or Integer literal"}, "123");
-      }
-    } else {
-      if (!PeekMatch(TokenType::Float)) {
+      } else if (!PeekMatch(TokenType::Float)) {
         return ErrorExpected({"a Float literal"}, "42.0");
       }
     }
@@ -1970,12 +1994,75 @@ Result WastParser::ParseConst(Const* const_) {
   return Result::Ok;
 }
 
+Result WastParser::ParseHostRef(Const* const_) {
+  WABT_TRACE(ParseHostRef);
+  Token token = Consume();
+  if (!options_->features.reference_types_enabled()) {
+    Error(token.loc, "hostref not allowed");
+    return Result::Error;
+  }
+
+  Literal literal;
+  string_view sv;
+  const char* s;
+  const char* end;
+  const_->loc = GetLocation();
+  TokenType token_type = Peek();
+
+  switch (token_type) {
+    case TokenType::Nat:
+    case TokenType::Int: {
+      literal = Consume().literal();
+      sv = literal.text;
+      s = sv.begin();
+      end = sv.end();
+      break;
+    }
+    default:
+      return ErrorExpected({"a numeric literal"}, "123");
+  }
+
+  uint64_t ref_bits;
+  Result result = ParseInt64(s, end, &ref_bits, ParseIntType::UnsignedOnly);
+
+  const_->type = ref_bits ? Type::Anyref : Type::Nullref;
+  const_->ref_bits = static_cast<uintptr_t>(ref_bits);
+
+  if (Failed(result)) {
+    Error(const_->loc, "invalid literal \"" PRIstringview "\"",
+          WABT_PRINTF_STRING_VIEW_ARG(literal.text));
+    // Return if parser get errors.
+    return Result::Error;
+  }
+
+  return Result::Ok;
+}
+
 Result WastParser::ParseConstList(ConstVector* consts) {
   WABT_TRACE(ParseConstList);
-  while (PeekMatchLpar(TokenType::Const)) {
+  while (PeekMatchLpar(TokenType::Const) || PeekMatchLpar(TokenType::RefNull) ||
+         PeekMatchLpar(TokenType::RefHost)) {
     Consume();
     Const const_;
-    CHECK_RESULT(ParseConst(&const_));
+    switch (Peek()) {
+      case TokenType::Const:
+        CHECK_RESULT(ParseConst(&const_));
+        break;
+      case TokenType::RefNull: {
+        auto token = Consume();
+        ErrorUnlessOpcodeEnabled(token);
+        const_.loc = GetLocation();
+        const_.type = Type::Nullref;
+        const_.ref_bits = 0;
+        break;
+      }
+      case TokenType::RefHost:
+        CHECK_RESULT(ParseHostRef(&const_));
+        break;
+      default:
+        assert(!"unreachable");
+        return Result::Error;
+    }
     EXPECT(Rpar);
     consts->push_back(const_);
   }
@@ -2377,6 +2464,7 @@ Result WastParser::ParseModuleCommand(Script* script, CommandPtr* out_command) {
     case ScriptModuleType::Binary: {
       auto* bsm = cast<BinaryScriptModule>(script_module.get());
       ReadBinaryOptions options;
+      options.features = options_->features;
       Errors errors;
       const char* filename = "<text>";
       ReadBinaryIr(filename, bsm->data.data(), bsm->data.size(), options,
@@ -2573,6 +2661,7 @@ Result ParseWatModule(WastLexer* lexer,
                       Errors* errors,
                       WastParseOptions* options) {
   assert(out_module != nullptr);
+  assert(options != nullptr);
   WastParser parser(lexer, errors, options);
   return parser.ParseModule(out_module);
 }
@@ -2582,6 +2671,7 @@ Result ParseWastScript(WastLexer* lexer,
                        Errors* errors,
                        WastParseOptions* options) {
   assert(out_script != nullptr);
+  assert(options != nullptr);
   WastParser parser(lexer, errors, options);
   return parser.ParseScript(out_script);
 }
