@@ -31,6 +31,7 @@
 #include "src/common.h"
 #include "src/expr-visitor.h"
 #include "src/ir.h"
+#include "src/ir-util.h"
 #include "src/literal.h"
 #include "src/stream.h"
 
@@ -88,28 +89,13 @@ struct ExprTree {
   Index result_count;
 };
 
-struct Label {
-  Label(LabelType label_type,
-        const std::string& name,
-        const TypeVector& param_types,
-        const TypeVector& result_types)
-      : name(name),
-        label_type(label_type),
-        param_types(param_types),
-        result_types(result_types) {}
-
-  std::string name;
-  LabelType label_type;
-  TypeVector param_types;
-  TypeVector result_types;
-};
-
-class WatWriter {
+class WatWriter : ModuleContext {
  public:
-  WatWriter(Stream* stream, const WriteWatOptions& options)
-      : options_(options), stream_(stream) {}
+  WatWriter(Stream* stream, const WriteWatOptions& options,
+            const Module &module)
+      : ModuleContext(module), options_(options), stream_(stream) {}
 
-  Result WriteModule(const Module& module);
+  Result WriteModule();
 
  private:
   void Indent();
@@ -171,11 +157,6 @@ class WatWriter {
 
   class ExprVisitorDelegate;
 
-  Index GetLabelStackSize() { return label_stack_.size(); }
-  Label* GetLabel(const Var& var);
-  Index GetLabelArity(const Var& var);
-  Index GetFuncParamCount(const Var& var);
-  Index GetFuncResultCount(const Var& var);
   void PushExpr(const Expr* expr, Index operand_count, Index result_count);
   void FlushExprTree(const ExprTree& expr_tree);
   void FlushExprTreeVector(const std::vector<ExprTree>&);
@@ -190,13 +171,10 @@ class WatWriter {
   void WriteInlineImport(ExternalKind, Index);
 
   const WriteWatOptions& options_;
-  const Module* module_ = nullptr;
-  const Func* current_func_ = nullptr;
   Stream* stream_ = nullptr;
   Result result_ = Result::Ok;
   int indent_ = 0;
   NextChar next_char_ = NextChar::None;
-  std::vector<Label> label_stack_;
   std::vector<ExprTree> expr_tree_stack_;
   std::multimap<std::pair<ExternalKind, Index>, const Export*>
       inline_export_map_;
@@ -437,14 +415,13 @@ void WatWriter::WriteBeginBlock(LabelType label_type,
     Writef(" ;; label = @%" PRIindex, GetLabelStackSize());
   }
   WriteNewline(FORCE_NEWLINE);
-  label_stack_.emplace_back(label_type, block.label, block.decl.sig.param_types,
-                            block.decl.sig.result_types);
+  BeginBlock(label_type, block);
   Indent();
 }
 
 void WatWriter::WriteEndBlock() {
   Dedent();
-  label_stack_.pop_back();
+  EndBlock();
   WritePutsNewline(Opcode::End_Opcode.GetName());
 }
 
@@ -579,6 +556,7 @@ class WatWriter::ExprVisitorDelegate : public ExprVisitor::Delegate {
   Result OnTernaryExpr(TernaryExpr*) override;
   Result OnSimdLaneOpExpr(SimdLaneOpExpr*) override;
   Result OnSimdShuffleOpExpr(SimdShuffleOpExpr*) override;
+  Result OnLoadSplatExpr(LoadSplatExpr*) override;
 
  private:
   WatWriter* writer_;
@@ -875,7 +853,7 @@ Result WatWriter::ExprVisitorDelegate::OnCatchExpr(TryExpr* expr) {
   writer_->Dedent();
   writer_->WritePutsSpace(Opcode::Catch_Opcode.GetName());
   writer_->Indent();
-  writer_->label_stack_.back().label_type = LabelType::Catch;
+  writer_->SetTopLabelType(LabelType::Catch);
   writer_->WriteNewline(FORCE_NEWLINE);
   return Result::Ok;
 }
@@ -952,6 +930,11 @@ Result WatWriter::ExprVisitorDelegate::OnSimdShuffleOpExpr(
   return Result::Ok;
 }
 
+Result WatWriter::ExprVisitorDelegate::OnLoadSplatExpr(LoadSplatExpr* expr) {
+  writer_->WriteLoadStoreExpr<LoadSplatExpr>(expr);
+  return Result::Ok;
+}
+
 void WatWriter::WriteExpr(const Expr* expr) {
   WABT_TRACE(WriteExprList);
   ExprVisitorDelegate delegate(this);
@@ -966,231 +949,10 @@ void WatWriter::WriteExprList(const ExprList& exprs) {
   visitor.VisitExprList(const_cast<ExprList&>(exprs));
 }
 
-Label* WatWriter::GetLabel(const Var& var) {
-  if (var.is_name()) {
-    for (Index i = GetLabelStackSize(); i > 0; --i) {
-      Label* label = &label_stack_[i - 1];
-      if (label->name == var.name()) {
-        return label;
-      }
-    }
-  } else if (var.index() < GetLabelStackSize()) {
-    Label* label = &label_stack_[GetLabelStackSize() - var.index() - 1];
-    return label;
-  }
-  return nullptr;
-}
-
-Index WatWriter::GetLabelArity(const Var& var) {
-  Label* label = GetLabel(var);
-  if (!label) {
-    return 0;
-  }
-
-  return label->label_type == LabelType::Loop ? label->param_types.size()
-                                              : label->result_types.size();
-}
-
-Index WatWriter::GetFuncParamCount(const Var& var) {
-  const Func* func = module_->GetFunc(var);
-  return func ? func->GetNumParams() : 0;
-}
-
-Index WatWriter::GetFuncResultCount(const Var& var) {
-  const Func* func = module_->GetFunc(var);
-  return func ? func->GetNumResults() : 0;
-}
-
 void WatWriter::WriteFoldedExpr(const Expr* expr) {
   WABT_TRACE_ARGS(WriteFoldedExpr, "%s", GetExprTypeName(*expr));
-  switch (expr->type()) {
-    case ExprType::AtomicNotify:
-    case ExprType::AtomicRmw:
-    case ExprType::Binary:
-    case ExprType::Compare:
-    case ExprType::TableGrow:
-      PushExpr(expr, 2, 1);
-      break;
-
-    case ExprType::AtomicStore:
-    case ExprType::Store:
-    case ExprType::TableSet:
-      PushExpr(expr, 2, 0);
-      break;
-
-    case ExprType::Block:
-      PushExpr(expr, 0, cast<BlockExpr>(expr)->block.decl.sig.GetNumResults());
-      break;
-
-    case ExprType::Br:
-      PushExpr(expr, GetLabelArity(cast<BrExpr>(expr)->var), 1);
-      break;
-
-    case ExprType::BrIf: {
-      Index arity = GetLabelArity(cast<BrIfExpr>(expr)->var);
-      PushExpr(expr, arity + 1, arity);
-      break;
-    }
-
-    case ExprType::BrOnExn:
-      PushExpr(expr, 1, 1);
-      break;
-
-    case ExprType::BrTable:
-      PushExpr(expr, GetLabelArity(cast<BrTableExpr>(expr)->default_target) + 1,
-               1);
-      break;
-
-    case ExprType::Call: {
-      const Var& var = cast<CallExpr>(expr)->var;
-      PushExpr(expr, GetFuncParamCount(var), GetFuncResultCount(var));
-      break;
-    }
-
-    case ExprType::ReturnCall: {
-      const Var& var = cast<ReturnCallExpr>(expr)->var;
-      PushExpr(expr, GetFuncParamCount(var), GetFuncResultCount(var));
-      break;
-    }
-
-    case ExprType::CallIndirect: {
-      const auto* ci_expr = cast<CallIndirectExpr>(expr);
-      PushExpr(expr, ci_expr->decl.GetNumParams() + 1,
-               ci_expr->decl.GetNumResults());
-      break;
-    }
-
-    case ExprType::ReturnCallIndirect: {
-      const auto* rci_expr = cast<ReturnCallIndirectExpr>(expr);
-      PushExpr(expr, rci_expr->decl.GetNumParams() + 1,
-               rci_expr->decl.GetNumResults());
-      break;
-    }
-
-    case ExprType::Const:
-    case ExprType::GlobalGet:
-    case ExprType::LocalGet:
-    case ExprType::MemorySize:
-    case ExprType::TableSize:
-    case ExprType::Unreachable:
-    case ExprType::RefNull:
-      PushExpr(expr, 0, 1);
-      break;
-
-    case ExprType::DataDrop:
-    case ExprType::ElemDrop:
-      PushExpr(expr, 0, 0);
-      break;
-
-    case ExprType::MemoryInit:
-    case ExprType::TableInit:
-    case ExprType::MemoryFill:
-    case ExprType::MemoryCopy:
-    case ExprType::TableCopy:
-      PushExpr(expr, 3, 0);
-      break;
-
-    case ExprType::AtomicLoad:
-    case ExprType::Convert:
-    case ExprType::Load:
-    case ExprType::LocalTee:
-    case ExprType::MemoryGrow:
-    case ExprType::Unary:
-    case ExprType::TableGet:
-    case ExprType::RefIsNull:
-      PushExpr(expr, 1, 1);
-      break;
-
-    case ExprType::Drop:
-    case ExprType::GlobalSet:
-    case ExprType::LocalSet:
-      PushExpr(expr, 1, 0);
-      break;
-
-    case ExprType::If:
-      PushExpr(expr, 1, cast<IfExpr>(expr)->true_.decl.sig.GetNumResults());
-      break;
-
-    case ExprType::Loop:
-      PushExpr(expr, 0, cast<LoopExpr>(expr)->block.decl.sig.GetNumResults());
-      break;
-
-    case ExprType::Nop:
-      PushExpr(expr, 0, 0);
-      break;
-
-    case ExprType::Return:
-      PushExpr(expr, current_func_->decl.sig.result_types.size(), 1);
-      break;
-
-    case ExprType::Rethrow:
-      PushExpr(expr, 0, 0);
-      break;
-
-    case ExprType::AtomicRmwCmpxchg:
-    case ExprType::AtomicWait:
-    case ExprType::Select:
-      PushExpr(expr, 3, 1);
-      break;
-
-    case ExprType::Throw: {
-      auto throw_ = cast<ThrowExpr>(expr);
-      Index operand_count = 0;
-      if (Event* event = module_->GetEvent(throw_->var)) {
-        operand_count = event->decl.sig.param_types.size();
-      }
-      PushExpr(expr, operand_count, 0);
-      break;
-    }
-
-    case ExprType::Try:
-      PushExpr(expr, 0, cast<TryExpr>(expr)->block.decl.sig.GetNumResults());
-      break;
-
-    case ExprType::Ternary:
-      PushExpr(expr, 3, 1);
-      break;
-
-    case ExprType::SimdLaneOp: {
-      const Opcode opcode = cast<SimdLaneOpExpr>(expr)->opcode;
-      switch (opcode) {
-        case Opcode::I8X16ExtractLaneS:
-        case Opcode::I8X16ExtractLaneU:
-        case Opcode::I16X8ExtractLaneS:
-        case Opcode::I16X8ExtractLaneU:
-        case Opcode::I32X4ExtractLane:
-        case Opcode::I64X2ExtractLane:
-        case Opcode::F32X4ExtractLane:
-        case Opcode::F64X2ExtractLane:
-          PushExpr(expr, 1, 1);
-          break;
-
-        case Opcode::I8X16ReplaceLane:
-        case Opcode::I16X8ReplaceLane:
-        case Opcode::I32X4ReplaceLane:
-        case Opcode::I64X2ReplaceLane:
-        case Opcode::F32X4ReplaceLane:
-        case Opcode::F64X2ReplaceLane:
-          PushExpr(expr, 2, 1);
-          break;
-
-        default:
-          fprintf(stderr, "Invalid Opcode for expr type: %s\n",
-                  GetExprTypeName(*expr));
-          assert(0);
-      }
-      break;
-    }
-
-    case ExprType::SimdShuffleOp:
-      PushExpr(expr, 2, 1);
-      break;
-
-    default:
-      fprintf(stderr, "bad expr type: %s\n", GetExprTypeName(*expr));
-      assert(0);
-      break;
-  }
+  auto arity = GetExprArity(*expr);
+  PushExpr(expr, arity.nargs, arity.nreturns);
 }
 
 void WatWriter::WriteFoldedExprList(const ExprList& exprs) {
@@ -1382,7 +1144,7 @@ void WatWriter::WriteBeginFunc(const Func& func) {
     WriteCloseSpace();
   }
 
-  if (module_->IsImport(ExternalKind::Func, Var(func_index_))) {
+  if (module.IsImport(ExternalKind::Func, Var(func_index_))) {
     // Imported functions can be written a few ways:
     //
     //   1. (import "module" "field" (func (type 0)))
@@ -1413,17 +1175,14 @@ void WatWriter::WriteFunc(const Func& func) {
                       func.GetNumParams());
   }
   WriteNewline(NO_FORCE_NEWLINE);
-  label_stack_.clear();
-  label_stack_.emplace_back(LabelType::Func, std::string(), TypeVector(),
-                            func.decl.sig.result_types);
-  current_func_ = &func;
+  BeginFunc(func);
   if (options_.fold_exprs) {
     WriteFoldedExprList(func.exprs);
     FlushExprTreeStack();
   } else {
     WriteExprList(func.exprs);
   }
-  current_func_ = nullptr;
+  EndFunc();
   WriteCloseNewline();
 }
 
@@ -1488,7 +1247,6 @@ void WatWriter::WriteElemSegment(const ElemSegment& segment) {
   WriteOpenSpace("elem");
   WriteNameOrIndex(segment.name, elem_segment_index_, NextChar::Space);
   if (segment.passive) {
-    WritePutsSpace("passive");
     WriteType(segment.elem_type, NextChar::Space);
   } else {
     WriteInitExpr(segment.offset);
@@ -1525,9 +1283,7 @@ void WatWriter::WriteMemory(const Memory& memory) {
 void WatWriter::WriteDataSegment(const DataSegment& segment) {
   WriteOpenSpace("data");
   WriteNameOrIndex(segment.name, data_segment_index_, NextChar::Space);
-  if (segment.passive) {
-    WritePutsSpace("passive");
-  } else {
+  if (!segment.passive) {
     WriteInitExpr(segment.offset);
   }
   WriteQuotedData(segment.data.data(), segment.data.size());
@@ -1600,8 +1356,7 @@ void WatWriter::WriteStartFunction(const Var& start) {
   WriteCloseNewline();
 }
 
-Result WatWriter::WriteModule(const Module& module) {
-  module_ = &module;
+Result WatWriter::WriteModule() {
   BuildInlineExportMap();
   BuildInlineImportMap();
   WriteOpenSpace("module");
@@ -1658,8 +1413,7 @@ void WatWriter::BuildInlineExportMap() {
     return;
   }
 
-  assert(module_);
-  for (Export* export_ : module_->exports) {
+  for (Export* export_ : module.exports) {
     Index index = kInvalidIndex;
 
     // Exported imports can't be written with inline exports, unless the
@@ -1671,29 +1425,29 @@ void WatWriter::BuildInlineExportMap() {
     //
     //   (func (export "e") (import "module" "field"))
     //
-    if (!options_.inline_import && module_->IsImport(*export_)) {
+    if (!options_.inline_import && module.IsImport(*export_)) {
       continue;
     }
 
     switch (export_->kind) {
       case ExternalKind::Func:
-        index = module_->GetFuncIndex(export_->var);
+        index = module.GetFuncIndex(export_->var);
         break;
 
       case ExternalKind::Table:
-        index = module_->GetTableIndex(export_->var);
+        index = module.GetTableIndex(export_->var);
         break;
 
       case ExternalKind::Memory:
-        index = module_->GetMemoryIndex(export_->var);
+        index = module.GetMemoryIndex(export_->var);
         break;
 
       case ExternalKind::Global:
-        index = module_->GetGlobalIndex(export_->var);
+        index = module.GetGlobalIndex(export_->var);
         break;
 
       case ExternalKind::Event:
-        index = module_->GetEventIndex(export_->var);
+        index = module.GetEventIndex(export_->var);
         break;
     }
 
@@ -1722,23 +1476,23 @@ bool WatWriter::IsInlineExport(const Export& export_) {
   Index index;
   switch (export_.kind) {
     case ExternalKind::Func:
-      index = module_->GetFuncIndex(export_.var);
+      index = module.GetFuncIndex(export_.var);
       break;
 
     case ExternalKind::Table:
-      index = module_->GetTableIndex(export_.var);
+      index = module.GetTableIndex(export_.var);
       break;
 
     case ExternalKind::Memory:
-      index = module_->GetMemoryIndex(export_.var);
+      index = module.GetMemoryIndex(export_.var);
       break;
 
     case ExternalKind::Global:
-      index = module_->GetGlobalIndex(export_.var);
+      index = module.GetGlobalIndex(export_.var);
       break;
 
     case ExternalKind::Event:
-      index = module_->GetEventIndex(export_.var);
+      index = module.GetEventIndex(export_.var);
       break;
   }
 
@@ -1751,8 +1505,7 @@ void WatWriter::BuildInlineImportMap() {
     return;
   }
 
-  assert(module_);
-  for (const Import* import : module_->imports) {
+  for (const Import* import : module.imports) {
     inline_import_map_[static_cast<size_t>(import->kind())].push_back(import);
   }
 }
@@ -1780,8 +1533,8 @@ void WatWriter::WriteInlineImport(ExternalKind kind, Index index) {
 Result WriteWat(Stream* stream,
                 const Module* module,
                 const WriteWatOptions& options) {
-  WatWriter wat_writer(stream, options);
-  return wat_writer.WriteModule(*module);
+  WatWriter wat_writer(stream, options, *module);
+  return wat_writer.WriteModule();
 }
 
 }  // namespace wabt
